# This is free and unencumbered software released into the public domain.

"""The Haltia.AI Software Development Kit (SDK) for Python."""

import sys

assert sys.version_info >= (3, 9), "The Haltia.AI SDK for Python requires Python 3.9+"

from .lib import *
from .util import *

__all__ = [
    'abi',
    'lib',
    'util',
]



# This is free and unencumbered software released into the public domain.

"""The Haltia.AI Software Development Kit (SDK) for Python."""

from . import abi

def licensee() -> str:
    """
    Returns the name and email of who the loaded SDK build is licensed to.

    The licensee string format is "J. Random Hacker <jhacker@example.com>".
    """
    return abi.haiGetLicenseeString().decode('utf-8', 'replace')

def version() -> str:
    """
    Returns the full version string for the loaded SDK build.

    The version string format is "24.0.0-dev.123 (2024-01-31)".

    Note that this version is likely to be different from the Python SDK
    library version. The two are compatible so long as the two initial
    major components of the version string are equivalent for both.
    """
    return abi.haiGetVersionString().decode('utf-8', 'replace')



# This is free and unencumbered software released into the public domain.

"""The Haltia.AI Software Development Kit (SDK) for Python."""

from warnings import warn
from .lib import *

lib = load_library()

# const char* haiGetLicenseeString()
try:
    haiGetLicenseeString = lib.haiGetLicenseeString
    haiGetLicenseeString.restype = c_char_p
    haiGetLicenseeString.argtypes = ()
except AttributeError as err:
    warn(err, ImportWarning)

# const char* haiGetVersionString()
try:
    haiGetVersionString = lib.haiGetVersionString
    haiGetVersionString.restype = c_char_p
    haiGetVersionString.argtypes = ()
except AttributeError as err:
    warn(err, ImportWarning)

__all__ = [
    'haiGetLicenseeString',
    'haiGetVersionString',
]



# This is free and unencumbered software released into the public domain.

"""The Haltia.AI Software Development Kit (SDK) for Python."""

import os
import platform

from ctypes import *

MACOS_APP_PATH = 'Applications/Haltia.AI.app'
MACOS_APP_SUBPATH = 'Contents/Frameworks/HaltiaAI.framework/HaltiaAI'
MACOS_SDK_DYLIB = 'libHaltiaAI.dylib'

def load_library() -> CDLL:
    for path in paths():
        try:
            return cdll.LoadLibrary(path)
        except OSError as err:
            #print(err)
            pass
    raise ModuleNotFoundError("Unable to find the Haltia.AI SDK binaries")

def paths() -> list[str]:
    result = []
    sys = platform.system()

    if sys == 'Darwin':  # macOS
        if (path := os.getenv('HALTIA_SDK')) is not None:
            result.append(path)
        if (path := os.getenv('HALTIA_APP')) is not None:
            result.append(os.path.join(path, MACOS_APP_SUBPATH))
        if (path := os.getenv('HOME')) is not None:
            result.append(os.path.join(path, MACOS_APP_PATH, MACOS_APP_SUBPATH))
        result.append(os.path.join('/', MACOS_APP_PATH, MACOS_APP_SUBPATH))
        result.append(MACOS_SDK_DYLIB)

    elif sys == 'Linux':
        result.append('libHaltiaAI.so')

    elif sys == 'Windows':
        result.append('HaltiaAI.dll')

    return list(dict.fromkeys(result))  # remove duplicates



# This is free and unencumbered software released into the public domain.

"""Installation script for the 'haltia' package."""

# See: https://packaging.python.org/en/latest/discussions/setup-py-deprecated/
# See: https://packaging.python.org/en/latest/guides/modernize-setup-py-project/

from codecs import open
from os import path
from setuptools import find_packages, setup

def readfile(*filepath):
    with open(path.join(*filepath), encoding='utf-8') as f:
        return f.read()

PWD = path.abspath(path.dirname(__file__))

VERSION = readfile(PWD, 'VERSION').rstrip()
LONG_DESCRIPTION = readfile(PWD, 'README.md')

setup(
    name='haltia',
    version=VERSION,
    description="Haltia.AI Software Development Kit (SDK) for Python",
    long_description=LONG_DESCRIPTION,
    long_description_content_type='text/markdown',
    url='https://sdk.haltia.ai',
    author='Haltia.AI',
    author_email='support@haltia.ai',
    license='Public Domain',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'License :: Public Domain',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3 :: Only',
        'Topic :: Software Development :: Libraries',
    ],
    keywords='',
    project_urls={
        'Bug Tracker': 'https://github.com/HaltiaAI/haltia.py/issues',
        'Changelog': 'https://github.com/HaltiaAI/haltia.py/blob/master/CHANGES.md',
        'Documentation': 'https://github.com/HaltiaAI/haltia.py/blob/master/README.md',
        'Source Code': 'https://github.com/HaltiaAI/haltia.py',
    },
    packages=find_packages(where='src'),
    package_dir={'': 'src'},
    install_requires=[
        'know.py>=0.0.1',
    ],
    python_requires='~=3.9',
    extras_require={'test': ['pytest']},
)
